/**
 * Floater for Elementor
 * Customizable floating navigation for Elementor editor
 * Exclusively on https://1.envato.market/floater-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.7
 * @copyright       (C) 2018 - 2021 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

"use strict";

/**
 * Mdp floater main object
 * @type {{ floatNavigation: mdpFloater.floatNavigation, addAnimation: mdpFloater.addAnimation, addNavigation: mdpFloater.addNavigation }}
 */

const mdpFloater = {
    floatNavigation: function ( wrapperName ) {
        const $wpAdminBar = document.querySelector( '#wpadminbar' );
        const $navWrapper = document.querySelector( `.${wrapperName} .mdp-floater-elementor-navigation-wrapper` );
        const $customMenuItems = document.querySelectorAll( `.${wrapperName} .mdp-floater-elementor-custom-navigation-item` );
        const $wpMenuWrapper = document.querySelector( `.${wrapperName} .mdp-floater-elementor-wp-menu-wrapper` );
        const $wpMenuItems = document.querySelectorAll( `.${wrapperName} .mdp-floater-elementor-wp-menu > .mdp-floater-elementor-wp-menu-item` );
        const $wpSubmenuItems = document.querySelectorAll( `.${wrapperName} .mdp-floater-elementor-wp-menu-dropdown > .mdp-floater-elementor-wp-menu-item` );
        const $customMenuLinks = document.querySelectorAll( `.${wrapperName} .mdp-floater-elementor-custom-navigation-link` );
        const $menuWrapper = document.querySelector( `.${wrapperName} .mdp-floater-elementor-custom-navigation` );
        const menuDisplayingDesktop = document.querySelector( `.${wrapperName}` ).dataset.menuDisplaying;
        const menuDisplayingWidescreen = document.querySelector( `.${wrapperName}` ).dataset.menuDisplayingWidescreen;
        const menuDisplayingLaptop = document.querySelector( `.${wrapperName}` ).dataset.menuDisplayingLaptop;
        const menuDisplayingTabletExtra = document.querySelector( `.${wrapperName}` ).dataset.menuDisplayingTabletExtra;
        const menuDisplayingMobileExtra = document.querySelector( `.${wrapperName} ` ).dataset.menuDisplayingMobileExtra;
        const menuMobileDisplaying = document.querySelector( `.${wrapperName}` ).dataset.menuDisplayingMobile;
        const menuTabletDisplaying = document.querySelector( `.${wrapperName} ` ).dataset.menuDisplayingTablet;
        const toggleBtnClass = document.querySelector( `.${wrapperName}` ).dataset.toggleButtonClass;
        const toggleCloseBtnClass = document.querySelector( `.${wrapperName}` ).dataset.toggleCloseButtonClass;
        const menuType = document.querySelector( `.${wrapperName}` ).dataset.menuType;
        const hoverTitle = document.querySelector( `.${wrapperName}` ).dataset.menuTitleHover;
        const toggleAnimation = document.querySelector( `.${wrapperName}` ).dataset.toggleMenuAnimation;
        const toggleAnimationDuration = document.querySelector( `.${wrapperName}` ).dataset.toggleMenuAnimationDuration;
        const toggleAnimationEasing = document.querySelector( `.${wrapperName}` ).dataset.toggleMenuAnimationEasing;
        const toggleAnimationDelay = document.querySelector( `.${wrapperName}` ).dataset.toggleMenuAnimationDelay;
        const menuPosition = document.querySelector( `.${wrapperName}` ).dataset.menuPosition;
        const regex = /post=\d+|preview_id=\d+|elementor-preview=\d+/g;


        // add spacing if wp admin bar
        if ( $wpAdminBar !== null ) {
            if (  menuPosition === 'top-left' || menuPosition === 'top-center' || menuPosition === 'top-right'  ) {
                let wpAdminBarHeight = $wpAdminBar.offsetHeight;
                $navWrapper.style.top = wpAdminBarHeight + 'px';
                window.addEventListener('resize', () => {
                    wpAdminBarHeight = $wpAdminBar.offsetHeight;
                    $navWrapper.style.top = wpAdminBarHeight + 'px';
                });
            }
        }

        // set active class for custom menu links
        if ( $customMenuLinks !== null ) {
            const currentPage = document.URL;
            let currentPagePostId;
            if ( currentPage.match( regex ) !== null ) {
                currentPagePostId = currentPage.match( regex ).join( '' ).match( /\d+/g ).join( '' );
            } else {
                currentPagePostId = '';
            }

            $customMenuLinks.forEach( megaMenuLink => {
                const postId = megaMenuLink.dataset.postLinkId;
                if ( currentPagePostId !== '' && postId === currentPagePostId || megaMenuLink.href === currentPage ) {
                    megaMenuLink.parentElement.classList.add( 'mdp-floater-elementor-custom-menu-item-current' );
                }
            } );
        }

        // set menu hover
        if ( hoverTitle === 'yes' && menuType === 'custom' ) {
            this.menuHover( wrapperName, $menuWrapper, $customMenuItems, false, null, 'mdp-floater-elementor-custom-navigation-item-hover-active' );
        } else {
            this.menuHover( wrapperName, $wpMenuWrapper, $wpMenuItems, true, $wpSubmenuItems,  'mdp-floater-elementor-wp-menu-item-hover', true );
        }

        // set menu displaying for different breakpoints
        let menuDisplaying;

        function toggleMenu() {
            const $box = document.querySelector( `.${wrapperName}` );
            if ( menuDisplaying === 'toggle' ) {
                const toggleButtonsHtml = `
                <div class="mdp-floater-elementor-toggle-button mdp-floater-elementor-toggle">
                    <i class="${toggleBtnClass}"></i>
                </div>
                <div class="mdp-floater-elementor-close-toggle-button mdp-floater-elementor-toggle">
                    <i class="${toggleCloseBtnClass}"></i>
                </div>
            `;

                // function for resetting active toggle items
                function resetActiveToggleItems( $menuItems, hideClass ) {
                    $menuItems.forEach( menuItem => {
                        if ( toggleAnimation === 'one_by_one_displaying' ) {
                            menuItem.style.animation = `grow ${toggleAnimationDuration}s ${toggleAnimationEasing} both`;
                            menuItem.classList.add( `${hideClass}` );
                        } else {
                            menuItem.classList.add( `${hideClass}` );
                        }
                    } );
                }

                // resetting active items
                if ( !$box.classList.contains( 'mdp-floater-elementor-toggle-menu-opened' ) ) {
                    resetActiveToggleItems( menuType === 'custom' ? $customMenuItems : $wpMenuItems, 'mdp-floater-elementor-hide' );
                }

                // add toggle buttons
                if ( document.querySelector( `.${wrapperName} .mdp-floater-elementor-toggle` ) === null ) {
                    $navWrapper.insertAdjacentHTML( 'afterbegin', toggleButtonsHtml );
                }

                const $toggleButton = document.querySelector( `.${wrapperName} .mdp-floater-elementor-toggle-button` );
                const $closeToggleButton = document.querySelector( `.${wrapperName} .mdp-floater-elementor-close-toggle-button` );

                if ( $toggleButton !== null && $closeToggleButton !== null ) {

                    $navWrapper.style.pointerEvents = 'none';
                    $toggleButton.style.pointerEvents = 'auto';

                    if ( toggleAnimation !== 'one_by_one_displaying' ) {
                        this.addAnimation( $toggleButton, null, toggleAnimation, toggleAnimationDuration, toggleAnimationEasing, toggleAnimationDelay );
                        this.addAnimation( $closeToggleButton, null, toggleAnimation, toggleAnimationDuration, toggleAnimationEasing, toggleAnimationDelay );
                    }

                    if ( !$box.classList.contains( 'mdp-floater-elementor-toggle-menu-opened' ) ) {
                        $closeToggleButton.style.display = 'none';
                    }

                    // toggle buttons
                    $toggleButton.addEventListener('click', () => {
                        $box.classList.add( 'mdp-floater-elementor-toggle-menu-opened' );
                        $toggleButton.style.pointerEvents = 'auto';
                        $navWrapper.style.pointerEvents = 'auto';
                        if ( toggleAnimation === 'one_by_one_displaying' ) {
                            oneByOneAnimation(menuType === 'custom' ? $customMenuItems : $wpMenuItems, toggleAnimationDuration, toggleAnimationDelay );
                        } else {
                            this.addAnimation(null, menuType === 'custom' ? $customMenuItems : $wpMenuItems, toggleAnimation, toggleAnimationDuration, toggleAnimationEasing, toggleAnimationDelay );
                        }
                        $toggleButton.style.display = 'none';
                        $closeToggleButton.style.display = 'flex';
                    });

                    $closeToggleButton.addEventListener('click', () => {
                        $box.classList.remove( 'mdp-floater-elementor-toggle-menu-opened' );
                        $navWrapper.style.pointerEvents = 'none';
                        $closeToggleButton.style.pointerEvents = 'auto';
                        resetActiveToggleItems(menuType === 'custom' ? $customMenuItems : $wpMenuItems, 'mdp-floater-elementor-hide');
                        $closeToggleButton.style.display = 'none';
                        $toggleButton.style.display = 'flex';
                    });
                }

            } else {
                $box.classList.remove( 'mdp-floater-elementor-toggle-menu-opened' );
                // set pointer events auto
                if ( $navWrapper ) {
                    $navWrapper.style.pointerEvents = 'auto';
                }

                // remove toggle buttons if not toggle menu
                const toggleButtons = document.querySelectorAll( `.${wrapperName} .mdp-floater-elementor-toggle` );
                const hiddenItems = document.querySelectorAll( `.${wrapperName} .mdp-floater-elementor-hide` );
                toggleButtons.forEach( toggleButton => {
                    toggleButton.remove();
                } );

                // remove hide class if not toggle menu
                if ( hiddenItems.length > 0 ) {
                    hiddenItems.forEach( item => {
                        item.classList.remove( 'mdp-floater-elementor-hide' );
                    } );
                }
            }
        }

        function checkScreenSize() {

            if ( window.matchMedia( `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.mobile.value}px)` ).matches ) {

                menuDisplaying = menuMobileDisplaying;

            } else if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'mobile_extra' )
                && window.matchMedia( ` (max-width: ${elementorFrontend.config.responsive.activeBreakpoints.mobile_extra.value}px) ` ).matches
                && menuDisplayingMobileExtra ) {

                menuDisplaying = menuDisplayingMobileExtra;

            } else if ( window.matchMedia( `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.tablet.value}px)` ).matches ) {

                menuDisplaying = menuTabletDisplaying;

            } else if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'tablet_extra' )
                && window.matchMedia( `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.tablet_extra.value}px) ` ).matches
                && menuDisplayingTabletExtra ) {

                menuDisplaying = menuDisplayingTabletExtra;

            } else if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'laptop' )
                && window.matchMedia( `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.laptop.value}px)` ).matches
                && menuDisplayingLaptop ) {

                menuDisplaying = menuDisplayingLaptop;


            } else if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'widescreen' )
                && window.matchMedia( `(min-width: ${elementorFrontend.config.responsive.activeBreakpoints.widescreen.value}px)` ).matches
                && menuDisplayingWidescreen ){

                menuDisplaying = menuDisplayingWidescreen;

            } else if ( menuDisplayingWidescreen && elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'widescreen' ) ) {
                if ( window.matchMedia( `(min-width: ${elementorFrontend.config.responsive.activeBreakpoints.tablet.value + 1}px)` ).matches
                    && window.matchMedia( `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.widescreen.value - 1}px)` ).matches ) {

                    menuDisplaying = menuDisplayingDesktop;

                }
            } else {
                if ( window.matchMedia( `(min-width: ${elementorFrontend.config.responsive.activeBreakpoints.tablet.value + 1}px)` ).matches ) {

                    menuDisplaying = menuDisplayingDesktop;

                }
            }

            toggleMenu.call( mdpFloater );
        }

        checkScreenSize();

        toggleMenu.call( mdpFloater );

        window.addEventListener( 'resize', () => {
            checkScreenSize();
        } );

        function oneByOneAnimation( $elements, duration, delay ) {
            setTimeout( () => {
            let count = 0;
            $elements[count].style.transform = 'scale(1)';
            $elements[count].classList.remove( 'mdp-floater-elementor-hide' );
            let timer = setInterval( () => {
                if ( count === $elements.length - 1 ) {
                    clearInterval( timer );
                    timer = null;
                } else {
                    count++;
                    $elements[count].style.transform = 'scale(1)';
                    $elements[count].classList.remove( 'mdp-floater-elementor-hide' );
                }
              },  +duration * 1000 );
            }, +delay * 1000 );
        }
    },

    menuHover: function ( wrapperName ,menuWrapper, menuItems, includeSubmenuItems, submenuItems, activeItemClass, wpMenu = false ) {
        let submenuEventListener = null;

        function resetActiveItems() {
            if ( menuWrapper !== null ) {
                const activeItems = menuWrapper.querySelectorAll( `.${activeItemClass}` );

                activeItems.forEach( item => {
                    item.classList.remove( `${activeItemClass}` );
                } );
                const activeDropdowns = document.querySelectorAll( `.${wrapperName} .mdp-floater-elementor-wp-menu-dropdown-active` );
                if ( activeDropdowns ) {
                    activeDropdowns.forEach( activeDropdown => {
                        activeDropdown.classList.remove( 'mdp-floater-elementor-wp-menu-dropdown-active' );
                    } );
                }
            }

        }

        //check if element has parent
        function isDescendant( parent, child ) {
            let node = child.parentNode;
            while ( node !== null ) {
                if( node === parent ) {
                    return true;
                }
                node = node.parentNode;
            }
            return false;
        }

        document.addEventListener( 'click', ( e ) => {
            if ( !isDescendant( menuWrapper, e.target ) ) {
                resetActiveItems();
            }
        } );

        function listenerMouseEnter( e ) {
            resetActiveItems();
            e.target.classList.add( `${activeItemClass}` );
        }

        if ( includeSubmenuItems ) {
            submenuEventListener = function ( e ) {
                if ( e.relatedTarget.classList.contains( 'mdp-floater-elementor-wp-menu-dropdown' ) ) {
                    e.relatedTarget.querySelectorAll( `.${activeItemClass}` ).forEach( item => {
                        item.classList.remove( `${activeItemClass}` );
                    } );
                }

                e.target.classList.add( `${activeItemClass}` );

                e.target.addEventListener( 'mouseleave', ( e ) => {
                    if ( e.relatedTarget !== null ) {
                        if ( e.relatedTarget.classList.contains( 'mdp-floater-elementor-wp-menu-item' )
                            ||  e.relatedTarget.classList.contains( 'mdp-floater-elementor-wp-menu-dropdown-link' )
                            ||  e.relatedTarget.classList.contains( 'mdp-floater-wp-menu-submenu-indicator' )
                            ||  e.relatedTarget.classList.contains( 'mdp-floater-elementor-wp-menu-link' ) ) {
                            e.target.classList.remove( `${activeItemClass}` );
                        } else if ( e.relatedTarget.classList.contains( 'mdp-floater-elementor-wp-menu-dropdown' ) ) {
                            e.relatedTarget.querySelectorAll( `.${activeItemClass}` ).forEach( item => {
                                item.classList.remove( `${activeItemClass}` );
                            } );
                            e.target.classList.add( `${activeItemClass}` );
                        }
                    }
                } );
            }

            submenuItems.forEach( submenuItem => {
                submenuItem.addEventListener( 'mouseenter', submenuEventListener );
            } );
        }

        menuItems.forEach( menuItem => {
            menuItem.addEventListener( 'mouseenter', listenerMouseEnter );
        } );

        if ( wpMenu ) {
            // wp submenu opening on touch devices
            const $wpIndicators = document.querySelectorAll(`.${wrapperName} .mdp-floater-wp-menu-submenu-indicator`);
            const $wpSubmenuIndicators = document.querySelectorAll(`.${wrapperName} .mdp-floater-elementor-wp-menu-dropdown .mdp-floater-wp-menu-submenu-indicator`);

            const menuTouchCallback = e => {
                e.preventDefault();
                resetActiveItems();
                e.target.parentElement.classList.add(`${activeItemClass}`);
            };

            const submenuTouchCallback = e => {
                e.preventDefault();
                if ( e.target.parentElement.classList.contains( 'mdp-floater-elementor-wp-menu-dropdown' ) ) {
                    e.target.parentElement.querySelectorAll( `.${activeItemClass}` ).forEach(item => {
                        item.classList.remove(`${activeItemClass}` );
                    } );
                }

                e.target.parentElement.classList.add(`${activeItemClass}`);
            };

                $wpIndicators.forEach(indicator => {
                    indicator.ontouchstart = menuTouchCallback;
                });

                $wpSubmenuIndicators.forEach(indicator => {
                    indicator.ontouchstart = submenuTouchCallback;
                } );


        }
    },

    addAnimation: function ( $element = null, $elements = null, animation, animationDuration, animationEasing, animationDelay ) {

        if ( $element !== null ) {
            $element.style.animation = `${animation} ${animationDuration}s ${animationEasing} ${animationDelay}s both`;
        } else {
            $elements.forEach( element => {
                element.classList.remove( 'mdp-floater-elementor-hide' );
                element.style.animation = `${animation} ${animationDuration}s ${animationEasing}  ${animationDelay}s both`;
            } );
        }

    },

    addNavigation: function () {
        const floaterBox = document.querySelectorAll( '.mdp-floater-elementor-box' );

        for ( let i = 0; i < floaterBox.length; i++ ) {
            floaterBox[i].classList.add( 'mdp-floater-elementor-box-' + i );
            this.floatNavigation( 'mdp-floater-elementor-box-' + i );
        }
    }
};

/**
 * Init for Front-End
 * @param callback
 */
document.addEventListener( 'DOMContentLoaded', mdpFloater.addNavigation.bind( mdpFloater ) );