<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Floater for Elementor
 * Customizable floating navigation for Elementor editor
 * Exclusively on https://1.envato.market/floater-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.0
 * @copyright       (C) 2018 - 2021 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\FloaterElementor;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Group_Control_Typography;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Repeater;
use Elementor\Icons_Manager;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\FloaterElementor\Unity\Plugin as UnityPlugin;
use Elementor\Plugin;
use Merkulove\FloaterElementor\Unity\ElementorControls;

/** @noinspection PhpUnused */
/**
 * Floater - Custom Elementor Widget.
 **/
class floater_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
              'mdp-floater-elementor-admin',
                 UnityPlugin::get_url() . 'src/Merkulove/Unity/assets/css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
                     [],
                     UnityPlugin::get_version()
        );
        wp_register_style(
             'mdp-floater-elementor',
                UnityPlugin::get_url() . 'css/floater-elementor' . UnityPlugin::get_suffix() . '.css',
                    [],
                    UnityPlugin::get_version()
        );
	    wp_register_script(
            'mdp-floater',
               UnityPlugin::get_url() . 'js/floater-elementor' . UnityPlugin::get_suffix() . '.js',
                   [ 'jquery' ],
                   UnityPlugin::get_version(),
          true
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-floater-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Floater', 'floater-elementor' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-floater-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'general' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Floater' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-floater-elementor', 'mdp-floater-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-floater' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> General Content Section. */
        $this->section_general_settings();

        /** Content -> Custom Navigation Section. */
        $this->section_custom_navigation_settings();

        /** Content -> Custom Navigation Item Section. */
        $this->section_custom_navigation_item_settings();

        /** Content -> Custom Wp Menu Settings Section. */
        $this->section_wp_menu_settings();

        /** Content -> Custom Wp Menu Submenu Section. */
        $this->section_wp_menu_submenu_settings();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Navigation Wrapper Styles. */
        $this->section_navigation_wrapper();

        /** Style -> Section Custom Menu Item Styles. */
        $this->section_custom_menu_item_styles();

        /** Style -> Section Custom Block Title Styles. */
        $this->section_custom_menu_block_title_styles();

        /** Style -> Section WP Menu Item Styles. */
        $this->section_wp_menu_item_styles();

        /** Style -> Section WP Menu Submenu Styles. */
        $this->section_wp_menu_submenu_styles();

        /** Style -> Section WP Menu Submenu Item Styles. */
        $this->section_wp_menu_submenu_item_styles();

        /** Style -> Section Toggle Styles. */
        $this->section_style_toggle();

        /** Style -> Section Open Toggle Styles. */
        $this->section_style_open_toggle();

        /** Style -> Section Close Toggle Styles. */
        $this->section_style_close_toggle();

    }

    /**
     * Get menus from wp
     *
     * @since 1.0.0
     * @access private
     *
     * @return array
     * */
    private function get_menus() {
        $menus = wp_get_nav_menus();

        $options = [];

        foreach ( $menus as $menu ) {
            $options[ $menu->slug ] = $menu->name;
        }

        return $options;
    }

    /**
     * Add widget controls: Content -> General Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_general_settings() {

        $this->start_controls_section( 'section_general', [
            'label' => esc_html__( 'General', 'floater-elementor' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'menu_type',
            [
                'label' => esc_html__( 'Menu', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'custom',
                'options' => [
                    'custom' => esc_html__( 'Custom', 'floater-elementor' ),
                    'wp_menu' => esc_html__( 'WP Menu', 'floater-elementor' ),
                ],
            ]
        );

        // get menus from wp
        $menus = $this->get_menus();

        if( !empty( $menus ) ) {
            $this->add_control(
                'select_menu',
                [
                    'label' => esc_html__( 'Select menu', 'floater-elementor' ),
                    'type' => Controls_Manager::SELECT,
                    'options' => $menus,
                    'default' => array_keys( $menus )[0],
                    'save_default' => true,
                    'condition' => [
                            'menu_type' => 'wp_menu'
                    ]
                ]
            );
        } else {
            $this->add_control(
                'menu_link',
                [
                    'type' => Controls_Manager::RAW_HTML,
                    'raw' => '<strong>' . esc_html__( 'There are no menus in your site.', 'floater-elementor' ) .
                        '</strong><br>' . sprintf( wp_kses_post( 'Go to the <a href="%s" target="_blank">Menus screen</a> to create one.' ),
                            admin_url( 'nav-menus.php?action=edit&menu=0' ) ),
                    'separator' => 'after',
                    'content_classes' => 'elementor-panel-alert elementor-panel-alert-info',
                    'condition' => [
                        'menu_type' => 'wp_menu'
                    ]
                ]
            );
        }

        $this->add_control(
            'menu_position',
            [
                'label' => esc_html__( 'Menu position', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'top-center',
                'options' => [
                    'top-left' => esc_html__( 'Top Left', 'floater-elementor' ),
                    'top-center' => esc_html__( 'Top Center', 'floater-elementor' ),
                    'top-right' => esc_html__( 'Top Right', 'floater-elementor' ),
                    'left-center' => esc_html__( 'Left Center', 'floater-elementor' ),
                    'right-center' => esc_html__( 'Right Center', 'floater-elementor' ),
                    'bottom-left' => esc_html__( 'Bottom Left', 'floater-elementor' ),
                    'bottom-center' => esc_html__( 'Bottom Center', 'floater-elementor' ),
                    'bottom-right' => esc_html__( 'Bottom Right', 'floater-elementor' ),
                ],
            ]
        );

        $this->add_control(
            'menu_layout',
            [
                'label' => esc_html__( 'Layout', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'horizontal',
                'options' => [
                    'vertical' => esc_html__( 'Vertical', 'floater-elementor' ),
                    'horizontal' => esc_html__( 'Horizontal', 'floater-elementor' ),
                ],
            ]
        );

        $this->add_responsive_control(
            'menu_displaying',
            [
                'label' => esc_html__( 'Menu displaying', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'always_show',
                'options' => [
                    'always_show' => esc_html__( 'Always show', 'floater-elementor' ),
                    'toggle' => esc_html__( 'Toggle', 'floater-elementor' ),
                ],
            ]
        );

        $this->add_control(
            'menu_toggle',
            [
                'label' => esc_html__( 'Toggle', 'floater-elementor' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fa fa-bars',
                    'library' => 'solid',
                ],
            ]
        );

        $this->add_control(
            'menu_close_toggle',
            [
                'label' => esc_html__(  'Close toggle', 'floater-elementor' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-times',
                    'library' => 'solid',
                ]
            ]
        );

        $this->add_control(
            'toggle_menu_animation',
            [
                'label' => esc_html__( 'Toggle menu animation', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'label_block' => true,
                'default' => 'none',
                'options' => [
                    'grow' => esc_html__( 'Grow', 'floater-elementor' ),
                    'shrink' => esc_html__( 'Shrink', 'floater-elementor' ),
                    'slide-up' => esc_html__( 'Slide up', 'floater-elementor' ),
                    'slide-down' => esc_html__( 'Slide down', 'floater-elementor' ),
                    'fade' => esc_html__( 'Fade', 'floater-elementor' ),
                    'one_by_one_displaying' => esc_html__( 'One by one displaying', 'floater-elementor' ),
                    'none' => esc_html__( 'None', 'floater-elementor' ),
                ],
                'separator' => 'after',
            ]
        );

        $this->add_control(
            'toggle_menu_animation_easing',
            [
                'label' => esc_html__( 'Easing', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'ease',
                'options' => [
                    'ease' => esc_html__( 'Ease', 'floater-elementor' ),
                    'ease-in' => esc_html__( 'Ease-in', 'floater-elementor' ),
                    'ease-out' => esc_html__( 'Ease-out', 'floater-elementor' ),
                    'ease-in-out' => esc_html__( 'Ease-in-out', 'floater-elementor' ),
                    'linear' => esc_html__( 'Linear', 'floater-elementor' ),
                ],
                'condition' => [
                    'toggle_menu_animation!' => 'none',
                ],
            ]
        );

        $this->add_control(
            'toggle_menu_animation_duration',
            [
                'label' => esc_html__( 'Duration', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 1,
                ],
                'condition' => [
                    'toggle_menu_animation!' => 'none',
                ],
            ]
        );

        $this->add_control(
            'toggle_menu_animation_delay',
            [
                'label' => esc_html__( 'Delay', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'condition' => [
                    'toggle_menu_animation!' => 'none',
                ],
            ]
        );


        $this->end_controls_section();

    }


    /**
     * Add widget controls: Content -> Custom Navigation Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_custom_navigation_settings() {

        $this->start_controls_section( 'section_custom_navigation', [
            'label' => esc_html__( 'Custom navigation', 'floater-elementor' ),
            'tab'   => Controls_Manager::TAB_CONTENT,
            'condition' => [
                   'menu_type' => 'custom'
            ]
        ] );

        $this->add_control(
            'display_menu_title_hover',
            [
                'label' => esc_html__( 'Display menu title on hover', 'floater-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'floater-elementor' ),
                'label_off' => esc_html__( 'No', 'floater-elementor' ),
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->add_control(
            'display_menu_title',
            [
                'label' => esc_html__( 'Display menu title', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'inline',
                'options' => [
                    'inline' => esc_html__( 'Inline', 'floater-elementor' ),
                    'in-another-block' => esc_html__( 'In another block', 'floater-elementor' ),
                ],
            ]
        );

        $this->add_control(
            'hover_title_animation',
            [
                'label' => esc_html__( 'Hover title animation', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'none',
                'options' => [
                    'grow' => esc_html__( 'Grow', 'floater-elementor' ),
                    'shrink' => esc_html__( 'Shrink', 'floater-elementor' ),
                    'slide-up' => esc_html__( 'Slide up', 'floater-elementor' ),
                    'slide-down' => esc_html__( 'Slide down', 'floater-elementor' ),
                    'fade' => esc_html__( 'Fade', 'floater-elementor' ),
                    'none' => esc_html__( 'None', 'floater-elementor' ),
                ],
                'condition' => [
                        'display_menu_title_hover' => 'yes',
                        'display_menu_title' => 'in-another-block'
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-custom-navigation-title-absolute' => 'animation: {{VALUE}} ',
                ],
            ]
        );

        $this->add_control(
            'hover_title_animation_easing',
            [
                'label' => esc_html__( 'Easing', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'ease',
                'options' => [
                    'ease' => esc_html__( 'Ease', 'floater-elementor' ),
                    'ease-in' => esc_html__( 'Ease-in', 'floater-elementor' ),
                    'ease-out' => esc_html__( 'Ease-out', 'floater-elementor' ),
                    'ease-in-out' => esc_html__( 'Ease-in-out', 'floater-elementor' ),
                    'linear' => esc_html__( 'Linear', 'floater-elementor' ),
                ],
                'condition' => [
                    'display_menu_title_hover' => 'yes',
                    'display_menu_title' => 'in-another-block',
                    'hover_title_animation!' => 'none'
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-custom-navigation-title-absolute' => 'animation-timing-function: {{VALUE}} ',
                ],
            ]
        );

        $this->add_control(
            'hover_title_animation_duration',
            [
                'label' => esc_html__( 'Duration', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 1,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-custom-navigation-title-absolute' => 'animation-duration: {{SIZE}}{{UNIT}}',
                ],
                'condition' => [
                    'display_menu_title_hover' => 'yes',
                    'display_menu_title' => 'in-another-block',
                    'hover_title_animation!' => 'none'
                ],
            ]
        );

        $this->add_control(
            'hover_title_animation_delay',
            [
                'label' => esc_html__( 'Delay', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-custom-navigation-title-absolute' => 'animation-delay: {{SIZE}}{{UNIT}}',
                ],
                'condition' => [
                    'display_menu_title' => 'in-another-block',
                    'hover_title_animation!' => 'none'
                ],
            ]
        );

        $this->add_control(
            'expand_hover_title_position_horizontal',
            [
                'label' => esc_html__( 'Expand title position', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'bottom',
                'options' => [
                    'top' => esc_html__( 'Top', 'floater-elementor' ),
                    'bottom' => esc_html__( 'Bottom', 'floater-elementor' ),
                ],
                'condition' => [
                    'display_menu_title' => 'in-another-block',
                    'menu_layout' => 'horizontal'
                ],
            ]
        );

        $this->add_control(
            'expand_hover_title_position_vertical',
            [
                'label' => esc_html__( 'Expand hover title position', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'right',
                'options' => [
                    'right' => esc_html__( 'Right', 'floater-elementor' ),
                    'left' => esc_html__( 'Left', 'floater-elementor' ),
                ],
                'condition' => [
                    'display_menu_title' => 'in-another-block',
                    'menu_layout' => 'vertical'
                ],
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Custom Navigation Item Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_custom_navigation_item_settings() {

        $this->start_controls_section('section_custom_navigation_item', [
            'label' => esc_html__('Custom navigation item', 'floater-elementor'),
            'tab' => Controls_Manager::TAB_CONTENT,
            'condition' => [
                'menu_type' => 'custom'
            ]
        ]);

        $repeater = new Repeater();

        $repeater->add_control(
            'custom_menu_item_title',
            [
                'label' => esc_html__( 'Title', 'floater-elementor' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Item', 'floater-elementor' ),
                'placeholder' => esc_html__( 'Type your title here', 'floater-elementor' ),
            ]
        );

        $repeater->add_control(
            'custom_menu_item_link',
            [
                'label' => esc_html__( 'Link', 'floater-elementor' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'floater-elementor' ),
                'show_external' => true,
                'default' => [
                    'url' => '',
                    'is_external' => true,
                    'nofollow' => true,
                ],
            ]
        );

        $repeater->add_control(
            'custom_menu_item_icon',
            [
                'label' => esc_html__( 'Icon', 'floater-elementor' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-star',
                    'library' => 'solid',
                ],
            ]
        );

        $this->add_control(
            'custom_menu_items_list',
            [
                'label' => esc_html__( 'Menu items', 'floater-elementor' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'default' => [
                    [
                        'item_title' => esc_html__( 'Item', 'floater-elementor' )
                    ],
                    [
                        'item_title' => esc_html__( 'Item', 'floater-elementor' )
                    ],
                    [
                        'item_title' => esc_html__( 'Item', 'floater-elementor' )
                    ],
                ],
                'title_field' => '{{{ custom_menu_item_title }}}',
            ]
        );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Content -> WP Menu Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_wp_menu_settings() {
        $this->start_controls_section('section_wp_menu', [
            'label' => esc_html__('WP menu', 'floater-elementor'),
            'tab' => Controls_Manager::TAB_CONTENT,
            'condition' => [
                    'menu_type' => 'wp_menu'
            ]
        ] );

        $this->add_control(
            'wp_menu_pointer',
            [
                'label' => esc_html__( 'Pointer', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'none',
                'options' => [
                    'mdp-floater-elementor-wp-menu-item-underline' => esc_html__('Underline', 'floater-elementor'),
                    'mdp-floater-elementor-wp-menu-item-overline' => esc_html__( 'Overline', 'floater-elementor' ),
                    'mdp-floater-elementor-wp-menu-item-double-line' => esc_html__( 'Double line', 'floater-elementor' ),
                    'mdp-floater-elementor-wp-menu-item-framed' => esc_html__( 'Framed', 'floater-elementor' ),
                    'none' => esc_html__( 'None', 'floater-elementor' ),
                ],
            ]
        );
        $this->add_control(
            'wp_menu_submenu_indicator',
            [
                'label' => esc_html__( 'Submenu indicator', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'fa fa-chevron-down',
                'options' => [
                    'fa fa-chevron-down' => esc_html__( 'Chevron', 'floater-elementor' ),
                    'fas fa-angle-down' => esc_html__( 'Angle', 'floater-elementor' ),
                    'fas fa-plus' => esc_html__( 'Plus', 'floater-elementor' ),
                    'fa fa-circle' => esc_html__( 'Circle', 'floater-elementor' ),
                    'fa fa-minus' => esc_html__( 'Minus', 'floater-elementor' ),
                    'fas fa-square' => esc_html__( 'Square', 'floater-elementor' ),
                    'custom' => esc_html__( 'Custom', 'floater-elementor' ),
                    '' => esc_html__( 'None', 'floater-elementor' )
                ],
            ]
        );


        $this->add_control(
            'wp_menu_custom_submenu_indicator',
            [
                'label' => esc_html__( 'Custom indicator', 'floater-elementor' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fa fa-chevron-down',
                    'library' => 'solid',
                ],
                'condition' => [ 'wp_menu_submenu_indicator' => 'custom' ]
            ]
        );

        $this->end_controls_section();

    }


    /**
     * Add widget controls: Content -> WP Menu Submenu Settings.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_wp_menu_submenu_settings() {
        $this->start_controls_section( 'section_wp_menu_submenu_settings', [
            'label' => esc_html__( 'WP menu submenu', 'floater-elementor' ),
            'tab'   => Controls_Manager::TAB_CONTENT,
            'condition' => [
                'menu_type' => 'wp_menu'
            ]
        ] );

        $this->add_control(
            'expand_wp_menu_position_horizontal',
            [
                'label' => esc_html__( 'Expand menu position', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'bottom',
                'options' => [
                    'top' => esc_html__( 'Top', 'floater-elementor' ),
                    'bottom' => esc_html__( 'Bottom', 'floater-elementor' ),
                ],
                'condition' => [
                    'menu_layout' => 'horizontal'
                ],
            ]
        );

        $this->add_control(
            'expand_wp_menu_position_vertical',
            [
                'label' => esc_html__( 'Expand menu position', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'right',
                'options' => [
                    'right' => esc_html__( 'Right', 'floater-elementor' ),
                    'left' => esc_html__( 'Left', 'floater-elementor' ),
                ],
                'condition' => [
                    'menu_layout' => 'vertical'
                ],
            ]
        );

        $this->add_control(
            'wp_submenu_pointer',
            [
                'label' => esc_html__( 'Submenu pointer', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'none',
                'separator' => 'before',
                'options' => [
                    'mdp-floater-elementor-wp-menu-submenu-item-underline' => esc_html__( 'Underline', 'floater-elementor' ),
                    'mdp-floater-elementor-wp-menu-submenu-item-overline' => esc_html__( 'Overline', 'floater-elementor' ),
                    'mdp-floater-elementor-wp-menu-submenu-item-double-line' => esc_html__( 'Double line', 'floater-elementor' ),
                    'mdp-floater-elementor-wp-menu-submenu-item-framed' => esc_html__( 'Framed', 'floater-elementor' ),
                    'none' => esc_html__( 'None', 'floater-elementor' ),
                ],
            ]
        );

        $this->add_control(
            'wp_submenu_animation',
            [
                'label' => esc_html__( 'Animation', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'none',
                'options' => [
                    'grow' => esc_html__( 'Grow', 'floater-elementor' ),
                    'shrink' => esc_html__( 'Shrink', 'floater-elementor' ),
                    'slide-up' => esc_html__( 'Slide up', 'floater-elementor' ),
                    'slide-down' => esc_html__( 'Slide down', 'floater-elementor' ),
                    'fade' => esc_html__( 'Fade', 'floater-elementor' ),
                    'none' => esc_html__( 'None', 'floater-elementor' ),
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-wp-menu-dropdown' => 'animation: {{VALUE}} ',
                ],
            ]
        );

        $this->add_control(
            'wp_menu_submenu_animation_easing',
            [
                'label' => esc_html__( 'Easing', 'floater-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'ease',
                'options' => [
                    'ease' => esc_html__( 'Ease', 'floater-elementor' ),
                    'ease-in' => esc_html__( 'Ease-in', 'floater-elementor' ),
                    'ease-out' => esc_html__( 'Ease-out', 'floater-elementor' ),
                    'ease-in-out' => esc_html__( 'Ease-in-out', 'floater-elementor' ),
                    'linear' => esc_html__( 'Linear', 'floater-elementor' ),
                ],
                'condition' => [
                    'wp_submenu_animation!' => 'none'
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-wp-menu-dropdown' => 'animation-timing-function: {{VALUE}} ',
                ],
            ]
        );

        $this->add_control(
            'wp_menu_submenu_animation_duration',
            [
                'label' => esc_html__( 'Duration', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 1,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-wp-menu-dropdown' => 'animation-duration: {{SIZE}}{{UNIT}}',
                ],
                'condition' => [
                    'wp_submenu_animation!' => 'none'
                ],
            ]
        );

        $this->add_control(
            'wp_menu_submenu_animation_delay',
            [
                'label' => esc_html__( 'Delay', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    '{{WRAPPER}} . mdp-floater-elementor-wp-menu-dropdown' => 'animation-delay: {{SIZE}}{{UNIT}}',
                ],
                'condition' => [
                    'wp_submenu_animation!' => 'none'
                ],
            ]
        );

        $this->end_controls_section();
    }

    /**
     * Function for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @return void
     * @since 1.0.0
     * @access private
     *
     */
    private function generate_margin_padding_controls( $section_id, $html_class ) {
        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'floater-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'floater-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            $section_id.'_margin_padding_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );
    }

    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param $html_class
     * @param bool $include_color
     * @param bool $include_typography
     * @param bool $pointer_color
     * @param string $pointer_color_class
     * @param string $pointer_color_name
     * @param bool $include_bg
     * @param bool $active_tab
     * @param string $active_tab_class
     * @param string $pointer_color_condition
     * @param bool $wp_menu
     * @param string $special_color_class
     * @param string $active_color_class
     * @param string $aditional_color_selector
     * @param string $aditional_color_hover_selector
     * @param bool $include_bg_video
     * @param bool $include_additional_color
     * @param string $additional_color_class
     * @param string $additional_color_name
     * @param bool $include_transition
     * @param string $background_transition_selector
     * @param string $color_transition_selector
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $html_class, $include_color = true, $include_typography = true, $pointer_color = false,
                                                        $pointer_color_class = '', $pointer_color_name = '', $include_bg = true, $active_tab = true, $active_tab_class = '',
                                                        $pointer_color_condition = '', $wp_menu = false, $special_color_class = '', $active_color_class = '', $aditional_color_selector = '',
                                                        $aditional_color_hover_selector = '', $include_bg_video = true, $include_additional_color = false, $additional_color_class = '',
                                                        $additional_color_name = '', $include_transition = false, $transition_selector = ''  ) {

        $color_class = $special_color_class !== '' ? $special_color_class : $html_class;
        $background_types = $include_bg_video ? ['classic', 'gradient', 'video'] : ['classic', 'gradient'];

        if ( $include_typography ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__('Typography', 'floater-elementor'),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .$html_class",
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab( $section_id.'_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'floater-elementor' )] );

        if ( $include_color ) {

            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__('Color', 'floater-elementor'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .$color_class" => 'color: {{VALUE}}',
                        "$aditional_color_selector" => 'color: {{VALUE}}'
                    ],
                ]
            );

        }

        if ( $include_additional_color ) {
            $this->add_control(
                $section_id . '_normal_'.$additional_color_name.'_color',
                [
                    'label' => esc_html__($additional_color_name.' color', 'floater-elementor'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .$additional_color_class" => 'color: {{VALUE}}',
                    ],
                ]
            );
        }

        if ( $include_bg ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'floater-elementor'),
                    'types' => $background_types,
                    'selector' => "{{WRAPPER}} .$html_class",
                ]
            );

            $this->add_control(
                $section_id . '_separate_normal',
                [
                    'type' => Controls_Manager::DIVIDER,
                ]
            );
        }

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_normal',
                'label' => esc_html__( 'Border Type', 'floater-elementor' ),
                'selector' => "{{WRAPPER}} .$html_class",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'floater-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'floater-elementor' ),
                'selector' => "{{WRAPPER}} .$html_class",
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab( $section_id.'_hover_style_tab', ['label' => esc_html__( 'HOVER', 'floater-elementor' )] );

        if ( $include_color ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__('Color', 'floater-elementor'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .$color_class:hover" => 'color: {{VALUE}} !important',
                        "$aditional_color_hover_selector" => 'color: {{VALUE}} !important'
                    ],
                ]
            );
        }

        if ( $include_additional_color ) {
            $this->add_control(
                $section_id . '_hover_'.$additional_color_name.'_color',
                [
                    'label' => esc_html__($additional_color_name.' color', 'floater-elementor'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .$html_class:hover .$additional_color_class" => 'color: {{VALUE}}',
                    ],
                ]
            );
        }

        if ( $pointer_color ) {

            $selectors = $wp_menu ? [
                "{{WRAPPER}} .$pointer_color_class-underline > li:hover::after" => 'background-color: {{VALUE}}',
                "{{WRAPPER}} .$pointer_color_class-overline > li:hover::after" => 'background-color: {{VALUE}}',
                "{{WRAPPER}} .$pointer_color_class-double-line > li:hover::after" => 'background-color: {{VALUE}}',
                "{{WRAPPER}} .$pointer_color_class-double-line > li:hover::before" => 'background-color: {{VALUE}}',
                "{{WRAPPER}} .$pointer_color_class-framed > li:hover::before" => 'border-color: {{VALUE}}',
            ] : [
                "{{WRAPPER}} .$pointer_color_class-underline:hover::after" => 'background-color: {{VALUE}}',
                "{{WRAPPER}} .$pointer_color_class-overline:hover::after" => 'background-color: {{VALUE}}',
                "{{WRAPPER}} .$pointer_color_class-double-line:hover::after" => 'background-color: {{VALUE}}',
                "{{WRAPPER}} .$pointer_color_class-double-line:hover::before" => 'background-color: {{VALUE}}',
                "{{WRAPPER}} .$pointer_color_class-framed:hover::before" => 'border-color: {{VALUE}}',
            ];

            $this->add_control(
                $section_id . '_pointer_color',
                [
                    'label' => esc_html__('Pointer Color', 'floater-elementor'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'condition' => [
                        "$pointer_color_condition!" => [ 'none' ]
                    ],
                    'selectors' => $selectors,
                ]
            );

        }

        if ( $include_bg ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'floater-elementor'),
                    'types' => $background_types,
                    'selector' => "{{WRAPPER}} .$html_class:hover",
                ]
            );

            $this->add_control(
                $section_id.'_separate_hover',
                [
                    'type' => Controls_Manager::DIVIDER,
                ]
            );
        }

        if ( $include_transition ) {
            $this->add_control(
                $section_id.'_hover_transition_menu_items',
                [
                    'label' => esc_html__( 'Hover transition duration', 'floater-elementor' ),
                    'type' => Controls_Manager::SLIDER,
                    'size_units' => [ 's' ],
                    'range' => [
                        's' => [
                            'min' => 0.1,
                            'max' => 5,
                            'step' => 0.1,
                        ],
                    ],
                    'default' => [
                        'unit' => 's',
                        'size' => 0,
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .'.$transition_selector => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}};',
                    ],
                ]
            );
        }


        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_hover',
                'label' => esc_html__( 'Border Type', 'floater-elementor' ),
                'selector' => "{{WRAPPER}} .$html_class:hover",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'floater-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .$html_class:hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'floater-elementor' ),
                'selector' => "{{WRAPPER}} .$html_class:hover",
            ]
        );

        $this->end_controls_tab();

        if ( $active_tab ) {

            $active_color_class = $active_color_class === '' ? $active_tab_class : $active_color_class;

            $this->start_controls_tab($section_id . '_active_style_tab', ['label' => esc_html__('Active', 'floater-elementor')]);

            if ($include_color) {
                $this->add_control(
                    $section_id . '_active_color',
                    [
                        'label' => esc_html__('Color', 'floater-elementor'),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => Color::COLOR_3,
                        ],
                        'selectors' => [
                            "{{WRAPPER}} .$active_color_class" => 'color: {{VALUE}} ',
                        ],
                    ]
                );
            }


            if ( $include_additional_color ) {
                $this->add_control(
                    $section_id . '_active_'.$additional_color_name.'_color',
                    [
                        'label' => esc_html__($additional_color_name.' color', 'floater-elementor'),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => Color::COLOR_3,
                        ],
                        'selectors' => [
                            "{{WRAPPER}} .$active_tab_class .$additional_color_class" => 'color: {{VALUE}}',
                        ],
                    ]
                );
            }


            if ($include_bg) {

                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name' => $section_id . '_background_active',
                        'label' => esc_html__('Background type', 'floater-elementor'),
                        'types' => $background_types,
                        'selector' => "{{WRAPPER}} .$active_tab_class",
                    ]
                );

                $this->add_control(
                    $section_id . '_separate_active',
                    [
                        'type' => Controls_Manager::DIVIDER,
                    ]
                );
            }


            $this->add_group_control(
                Group_Control_Border::get_type(),
                [
                    'name' => $section_id . '_border_active',
                    'label' => esc_html__('Border Type', 'floater-elementor'),
                    'selector' => "{{WRAPPER}} .$active_tab_class",
                ]
            );

            $this->add_responsive_control(
                $section_id . '_border_radius_active',
                [
                    'label' => esc_html__('Border radius', 'floater-elementor'),
                    'type' => Controls_Manager::DIMENSIONS,
                    'size_units' => ['px', '%', 'em'],
                    'selectors' => [
                        "{{WRAPPER}} .$active_tab_class" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                ]
            );

            $this->add_group_control(
                Group_Control_Box_Shadow::get_type(),
                [
                    'name' => $section_id . '_box_shadow_active',
                    'label' => esc_html__('Box Shadow', 'floater-elementor'),
                    'selector' => "{{WRAPPER}} .$active_tab_class",
                ]
            );

            $this->end_controls_tab();
        }

        $this->end_controls_tabs();
    }


    /**
     * Add widget controls: Style -> Section Navigation Wrapper Styles.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_navigation_wrapper() {
        $this->start_controls_section( 'section_navigation_wrapper', [
            'label' => esc_html__( 'Navigation wrapper', 'floater-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
        ] );

        $this->generate_margin_padding_controls( 'section_navigation_wrapper', 'mdp-floater-elementor-navigation-wrapper' );

        $this->generate_typography_tabs_controls( 'section_navigation_wrapper', 'mdp-floater-elementor-navigation-wrapper', false, false,
            false, '', '', true, false, '', '', false, '', '',
            '', '', true, '', '', '', true, 'mdp-floater-elementor-navigation-wrapper' );

        $this->end_controls_section();


    }

    /**
     * Add widget controls: Style -> Section Custom Menu Item Styles.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_custom_menu_item_styles() {

        $this->start_controls_section( 'section_custom_menu_item', [
            'label' => esc_html__( 'Custom menu item', 'floater-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'menu_type' => 'custom'
            ]
        ] );

        $this->generate_margin_padding_controls( 'section_custom_menu_item', 'mdp-floater-elementor-custom-navigation-item' );

        $this->add_control(
            'items_spacing',
            [
                'label' => esc_html__( 'Items spacing', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 200,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-custom-navigation-item' => 'margin-right: calc({{SIZE}}{{UNIT}} / 2); margin-left: calc({{SIZE}}{{UNIT}} / 2);',
                    '{{WRAPPER}} .mdp-floater-elementor-custom-navigation-item:first-child' => 'margin-left: calc({{SIZE}}{{UNIT}} * 0);',
                    '{{WRAPPER}} .mdp-floater-elementor-custom-navigation-item:last-child' => 'margin-right: calc({{SIZE}}{{UNIT}} * 0);',
                    '{{WRAPPER}} .mdp-floater-elementor-navigation-layout-vertical > .mdp-floater-elementor-custom-navigation > .mdp-floater-elementor-custom-navigation-item' => 'margin-right: 0; margin-left: 0; margin-top: calc({{SIZE}}{{UNIT}} / 2); margin-bottom: calc({{SIZE}}{{UNIT}} / 2);',
                    '{{WRAPPER}} .mdp-floater-elementor-navigation-layout-vertical > .mdp-floater-elementor-custom-navigation > .mdp-floater-elementor-custom-navigation-item:first-child' => 'margin-top: calc({{SIZE}}{{UNIT}} * 0);',
                    '{{WRAPPER}} .mdp-floater-elementor-navigation-layout-vertical > .mdp-floater-elementor-custom-navigation > .mdp-floater-elementor-custom-navigation-item:last-child' => 'margin-bottom: calc({{SIZE}}{{UNIT}} * 0);',
                ]
            ]
        );


        $this->add_responsive_control(
            'title_width',
            [
                'label' => esc_html__( 'Title block width', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-custom-navigation-title' => 'width: {{SIZE}}{{UNIT}};',
                ],

            ]
        );
        $this->add_responsive_control(
            'icon_size',
            [
                'label' => esc_html__( 'Icon size', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 200,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 16,
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-custom-navigation-icon' => 'font-size: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}}; height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );



        $this->add_responsive_control(
            'icon_spacing',
            [
                'label' => esc_html__( 'Icon spacing', 'floater-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .mdp-floater-elementor-custom-navigation-icon" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'separate_item_spacing',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->generate_typography_tabs_controls( 'section_custom_menu_item', 'mdp-floater-elementor-custom-navigation > .mdp-floater-elementor-custom-navigation-item',
            true, true,
            false, '', '', true, true,
            'mdp-floater-elementor-custom-navigation > .mdp-floater-elementor-custom-menu-item-current', '' , false,
            'mdp-floater-elementor-custom-navigation  > .mdp-floater-elementor-custom-navigation-item span',
            'mdp-floater-elementor-custom-navigation > .mdp-floater-elementor-custom-menu-item-current > .mdp-floater-elementor-custom-navigation-link > span',
            '{{WRAPPER}} .mdp-floater-elementor-custom-navigation-item a span', '{{WRAPPER}} .mdp-floater-elementor-custom-navigation-item:hover span',
            false, true, 'mdp-floater-elementor-custom-navigation-icon', 'Icon', true,
            'mdp-floater-elementor-custom-navigation > .mdp-floater-elementor-custom-navigation-item, {{WRAPPER}} .mdp-floater-elementor-custom-navigation .mdp-floater-elementor-custom-navigation-title, {{WRAPPER}} .mdp-floater-elementor-custom-navigation-icon' );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Custom Menu Item Title In Another Block Styles.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_custom_menu_block_title_styles() {

        $this->start_controls_section( 'section_custom_menu_block_title_styles', [
            'label' => esc_html__( 'Block title styles', 'floater-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'display_menu_title' => 'in-another-block',
                'menu_type' => 'custom'
            ]
        ] );

        $this->generate_margin_padding_controls( 'section_custom_menu_block_title_styles', 'mdp-floater-elementor-custom-navigation-title-absolute' );

        $this->add_responsive_control(
            'block_title_offset_x',
            [
                'label' => esc_html__( 'Offset - X', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => -500,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-navigation-layout-horizontal .mdp-floater-elementor-custom-navigation-title-hover-expand-top .mdp-floater-elementor-custom-navigation-title-absolute, {{WRAPPER}} .mdp-floater-elementor-navigation-layout-horizontal .mdp-floater-elementor-custom-navigation-title-hover-expand-bottom .mdp-floater-elementor-custom-navigation-title-absolute' => 'left: {{SIZE}}{{UNIT}} !important;',
                    '{{WRAPPER}} .mdp-floater-elementor-navigation-layout-vertical .mdp-floater-elementor-custom-navigation-title-hover-expand-left  .mdp-floater-elementor-custom-navigation-title-absolute' => 'left: {{SIZE}}{{UNIT}}; right: auto !important;',
                    '{{WRAPPER}} .mdp-floater-elementor-navigation-layout-vertical .mdp-floater-elementor-custom-navigation-title-hover-expand-right .mdp-floater-elementor-custom-navigation-title-absolute' => 'right: {{SIZE}}{{UNIT}}; left: auto !important;'
                ],

            ]
        );


        $this->add_responsive_control(
            'block_title_offset_y',
            [
                'label' => esc_html__( 'Offset - Y', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => -150,
                        'max' => 150,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-custom-navigation-title-hover-expand-right .mdp-floater-elementor-custom-navigation-title-absolute, {{WRAPPER}} .mdp-floater-elementor-custom-navigation-title-hover-expand-left  .mdp-floater-elementor-custom-navigation-title-absolute' => 'top: {{SIZE}}{{UNIT}} !important; bottom: auto;',
                    '{{WRAPPER}} .mdp-floater-elementor-custom-navigation-title-hover-expand-top .mdp-floater-elementor-custom-navigation-title-absolute' => 'bottom: {{SIZE}}{{UNIT}} !important; top: auto;',
                    '{{WRAPPER}} .mdp-floater-elementor-custom-navigation-title-hover-expand-bottom .mdp-floater-elementor-custom-navigation-title-absolute' => 'top: {{SIZE}}{{UNIT}} !important; bottom: auto',
                ],
            ]
        );


        $this->add_control(
            'separate_item_offset',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->generate_typography_tabs_controls( 'section_custom_menu_block_title_styles', 'mdp-floater-elementor-custom-navigation-title-absolute', false, false,
            '', '', '', true, false, '', '', false,
            'mdp-floater-elementor-custom-navigation-item span', '', '',
            '', false, false, '', '', true,
            'mdp-floater-elementor-custom-navigation-title.mdp-floater-elementor-custom-navigation-title-absolute' );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section WP Menu Item Styles.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_wp_menu_item_styles() {

        $this->start_controls_section( 'section_wp_menu_item', [
            'label' => esc_html__( 'WP menu item', 'floater-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'menu_type' => 'wp_menu'
            ]
        ] );

        $this->generate_margin_padding_controls( 'section_wp_menu_item', 'mdp-floater-elementor-wp-menu-item' );

        $this->add_control(
            'submenu_indicator_color_in_wp_menu',
            [
                'label' => esc_html__( 'Indicator color', 'floater-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-wp-menu-submenu-indicator' => 'color: {{VALUE}}',
                ],
            ]
        );


        $this->add_responsive_control(
            'wp_menu_indicator_click_area',
            [
                'label' => esc_html__( 'Indicator click area', 'floater-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-wp-menu > .mdp-floater-elementor-wp-menu-item > .mdp-floater-wp-menu-submenu-indicator' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );


        $this->add_responsive_control(
            'wp_menu_indicator_spacing',
            [
                'label' => esc_html__( 'Indicator spacing', 'floater-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-wp-menu-submenu-indicator' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'wp_menu_submenu_indicator_size_in_menu',
            [
                'label' => esc_html__( 'Submenu indicator size', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 10,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 20,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-wp-menu-submenu-indicator' => 'font-size: {{SIZE}}{{UNIT}}'
                ],
            ]
        );

        $this->add_control(
            'separate_wp_item_indicator',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_responsive_control(
            'space_between_wp_menu_items',
            [
                'label' => esc_html__( 'Space between items', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-wp-menu > .mdp-floater-elementor-wp-menu-item' => 'margin-right: calc({{SIZE}}{{UNIT}} / 2); margin-left: calc({{SIZE}}{{UNIT}} / 2);',
                    '{{WRAPPER}} .mdp-floater-elementor-wp-menu > .mdp-floater-elementor-wp-menu-item:first-child' => 'margin-left: calc({{SIZE}}{{UNIT}} * 0);',
                    '{{WRAPPER}} .mdp-floater-elementor-wp-menu > .mdp-floater-elementor-wp-menu-item:last-child' => 'margin-right: calc({{SIZE}}{{UNIT}} * 0);',
                    '{{WRAPPER}} .mdp-floater-elementor-wp-menu-vertical > .mdp-floater-elementor-wp-menu-item' => 'margin-right: 0; margin-left: 0; margin-top: calc({{SIZE}}{{UNIT}} / 2); margin-bottom: calc({{SIZE}}{{UNIT}} / 2);',
                    '{{WRAPPER}} .mdp-floater-elementor-wp-menu-vertical > .mdp-floater-elementor-wp-menu-item:first-child' => 'margin-top: calc({{SIZE}}{{UNIT}} * 0);',
                    '{{WRAPPER}} .mdp-floater-elementor-wp-menu-vertical > .mdp-floater-elementor-wp-menu-item:last-child' => 'margin-bottom: calc({{SIZE}}{{UNIT}} * 0);',
                ]
            ]
        );

        $this->add_control(
            'separate_wp_item_spacing',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->generate_typography_tabs_controls( 'section_wp_menu_item', 'mdp-floater-elementor-wp-menu > .mdp-floater-elementor-wp-menu-item', true, true,
            true, 'mdp-floater-elementor-wp-menu-item', 'Pointer', true, true, 'mdp-floater-elementor-wp-menu > .mdp-floater-elementor-wp-menu-item-active', 'wp_menu_pointer', true,
            'mdp-floater-elementor-wp-menu > .mdp-floater-elementor-wp-menu-item > a',
            'mdp-floater-elementor-wp-menu > .mdp-floater-elementor-wp-menu-item-active > a', '',
            '{{WRAPPER}} .mdp-floater-elementor-wp-menu > .mdp-floater-elementor-wp-menu-item:hover > a', false,
            false, '', '', true,
            'mdp-floater-elementor-wp-menu > .mdp-floater-elementor-wp-menu-item, {{WRAPPER}} .mdp-floater-elementor-wp-menu > .mdp-floater-elementor-wp-menu-item > a' );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section WP Menu Submenu Styles.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_wp_menu_submenu_styles() {
        $this->start_controls_section( 'section_wp_menu_submenu', [
            'label' => esc_html__( 'WP menu submenu', 'floater-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'menu_type' => 'wp_menu'
            ]
        ] );

        $this->generate_margin_padding_controls( 'section_wp_menu_submenu', 'mdp-floater-elementor-wp-menu-dropdown' );

        $this->add_responsive_control(
            'submenu_offset_x',
            [
                'label' => esc_html__( 'Offset - X', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => -500,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1
                    ],
                ],

                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-navigation-layout-horizontal .mdp-floater-wp-menu-expand-position-top .mdp-floater-elementor-wp-menu-dropdown, {{WRAPPER}} .mdp-floater-elementor-navigation-layout-horizontal .mdp-floater-wp-menu-expand-position-bottom .mdp-floater-elementor-wp-menu-dropdown' => 'left: {{SIZE}}{{UNIT}} !important',
                    '{{WRAPPER}} .mdp-floater-elementor-navigation-layout-vertical .mdp-floater-wp-menu-expand-position-left  .mdp-floater-elementor-wp-menu-dropdown' => 'left: {{SIZE}}{{UNIT}} !important; right: auto !important;',
                    '{{WRAPPER}} .mdp-floater-elementor-navigation-layout-vertical .mdp-floater-wp-menu-expand-position-right .mdp-floater-elementor-wp-menu-dropdown' => 'right: {{SIZE}}{{UNIT}} !important; left: auto !important;'
                ],

            ]
        );


        $this->add_responsive_control(
            'submenu_offset_y',
            [
                'label' => esc_html__( 'Offset - Y', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => -150,
                        'max' => 150,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-wp-menu-expand-position-right .mdp-floater-elementor-wp-menu-dropdown, {{WRAPPER}} .mdp-floater-wp-menu-expand-position-left .mdp-floater-elementor-wp-menu-dropdown' => 'top: {{SIZE}}{{UNIT}} !important; bottom: auto;',
                    '{{WRAPPER}} .mdp-floater-wp-menu-expand-position-top .mdp-floater-elementor-wp-menu-dropdown' => 'bottom: {{SIZE}}{{UNIT}} !important; top: auto;',
                    '{{WRAPPER}} .mdp-floater-wp-menu-expand-position-bottom  .mdp-floater-elementor-wp-menu-dropdown' => 'top: {{SIZE}}{{UNIT}} !important; bottom: auto',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_wp_menu_submenu', 'mdp-floater-elementor-wp-menu-dropdown', false, false,
            false, '', '', true, false, '', '', false, '', '', '',
            '', false, false, '', '', true, 'mdp-floater-elementor-wp-menu-dropdown' );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section WP Menu Submenu Item Styles.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_wp_menu_submenu_item_styles() {

        $this->start_controls_section( 'section_wp_menu_submenu_item', [
            'label' => esc_html__( 'WP menu submenu item', 'floater-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'menu_type' => 'wp_menu'
            ]
        ] );

        $this->generate_margin_padding_controls( 'section_wp_menu_submenu_item', 'mdp-floater-elementor-wp-menu-dropdown .mdp-floater-elementor-wp-menu-item' );

        $this->add_control(
            'submenu_indicator_color_in_wp_menu_submenu',
            [
                'label' => esc_html__( 'Indicator color', 'floater-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-wp-menu-dropdown .mdp-floater-wp-menu-submenu-indicator' => 'color: {{VALUE}}',
                ],
            ]
        );


        $this->add_responsive_control(
            'wp_menu_submenu_indicator_click_area',
            [
                'label' => esc_html__( 'Indicator click area', 'floater-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-wp-menu-dropdown .mdp-floater-wp-menu-submenu-indicator' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );


        $this->add_responsive_control(
            'wp_menu_submenu_indicator_spacing',
            [
                'label' => esc_html__( 'Indicator spacing', 'floater-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-wp-menu-dropdown .mdp-floater-wp-menu-submenu-indicator' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'wp_menu_submenu_indicator_size_in_submenu',
            [
                'label' => esc_html__( 'Submenu indicator size', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 10,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 20,
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-wp-menu-dropdown .mdp-floater-wp-menu-submenu-indicator' => 'font-size: {{SIZE}}{{UNIT}}'
                ],
            ]
        );

        $this->add_control(
            'separate_wp_submenu_item_spacing',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->generate_typography_tabs_controls( 'section_wp_menu_submenu_item', 'mdp-floater-elementor-wp-menu-dropdown .mdp-floater-elementor-wp-menu-item',
            true, true, true, 'mdp-floater-elementor-wp-menu-submenu-item',
            'Pointer', true,  true, 'mdp-floater-elementor-wp-menu-dropdown > .mdp-floater-elementor-wp-menu-item-active',
            'wp_submenu_pointer', true, 'mdp-floater-elementor-wp-menu-dropdown .mdp-floater-elementor-wp-menu-item a',
            'mdp-floater-elementor-wp-menu-dropdown > .mdp-floater-elementor-wp-menu-item > a', '', '{{WRAPPER}} .mdp-floater-elementor-wp-menu-dropdown .mdp-floater-elementor-wp-menu-item:hover a',
            false, false, '', '', true,
            'mdp-floater-elementor-wp-menu-dropdown .mdp-floater-elementor-wp-menu-item, {{WRAPPER}} .mdp-floater-elementor-wp-menu-dropdown .mdp-floater-elementor-wp-menu-item a' );


        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Toggle Style.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_toggle() {
        $this->start_controls_section('section_style_toggle', [
            'label' => esc_html__('Toggle style', 'floater-elementor'),
            'tab' => Controls_Manager::TAB_STYLE,
        ] );

        $this->generate_margin_padding_controls( 'section_style_toggle', 'mdp-floater-elementor-toggle i' );

        $this->add_control(
            'toggle_size',
            [
                'label' => esc_html__( 'Toggle size', 'floater-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    's' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 20,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-floater-elementor-toggle i' => 'font-size: {{SIZE}}{{UNIT}}'
                ],
            ]
        );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section Open Toggle Style.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_open_toggle() {
        $this->start_controls_section('section_style_open_toggle', [
            'label' => esc_html__('Open toggle style', 'floater-elementor'),
            'tab' => Controls_Manager::TAB_STYLE,
        ] );

        $this->generate_typography_tabs_controls( 'section_style_open_toggle', 'mdp-floater-elementor-toggle-button i', true, false, '',
            '', '', true, false, '', '', false, '', '', '', '',
            false, false, '', '', true, 'mdp-floater-elementor-toggle-button i' );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section Close Toggle Style.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_close_toggle() {
        $this->start_controls_section('section_style_close_toggle', [
            'label' => esc_html__('Close toggle style', 'floater-elementor'),
            'tab' => Controls_Manager::TAB_STYLE,
        ] );

        $this->generate_typography_tabs_controls( 'section_style_close_toggle', 'mdp-floater-elementor-close-toggle-button i', true,
            false, '', '', '', true, false, false, '', false, '', '', '',
            '', false, false, false, '', true, 'mdp-floater-elementor-close-toggle-button i' );


        $this->end_controls_section();
    }




    /**
     * Custom menu.
     *
     * @param $settings
     *
     * @return void
     */
    private function render_custom_menu( $settings ) {
        ?>
            <nav class="mdp-floater-elementor-custom-navigation <?php
            if ( $settings['display_menu_title'] === 'in-another-block' && $settings['menu_layout'] === 'vertical' ): ?>
                mdp-floater-elementor-custom-navigation-title-hover-expand-<?php esc_attr_e( $settings['expand_hover_title_position_vertical'] ); ?>
            <?php elseif ( $settings['display_menu_title'] === 'in-another-block' && $settings['menu_layout'] === 'horizontal' ): ?>
                mdp-floater-elementor-custom-navigation-title-hover-expand-<?php esc_attr_e( $settings['expand_hover_title_position_horizontal'] ); ?>
            <?php endif; ?> ">
                <?php foreach ( $settings['custom_menu_items_list'] as $index => $item ): ?>
                    <div class="mdp-floater-elementor-custom-navigation-item <?php if ( $settings['display_menu_title_hover'] === 'yes' ): ?>mdp-floater-elementor-custom-navigation-title-hover<?php endif; ?>">
                        <?php if ( !empty( $item['custom_menu_item_link']['url'] ) ): ?>
                            <a <?php
                                $this->add_link_attributes( 'custom_menu_item_link_' . $index, $item['custom_menu_item_link'] );
                                $this->print_render_attribute_string( 'custom_menu_item_link_' . $index ); ?>
                               class="mdp-floater-elementor-custom-navigation-link"
                               data-post-link-id="<?php esc_attr_e( url_to_postid( $item['custom_menu_item_link']['url'] ) !== 0 ? url_to_postid( $item['custom_menu_item_link']['url'] ) : '' ); ?>">
                                <div class="mdp-floater-elementor-custom-navigation-icon">
                                    <?php Icons_Manager::render_icon( $item['custom_menu_item_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                                </div>
                                <span class="mdp-floater-elementor-custom-navigation-title
                                    <?php if ( $settings['display_menu_title'] === 'in-another-block' ): ?> mdp-floater-elementor-custom-navigation-title-absolute <?php endif; ?>">
                                    <?php esc_html_e( $item['custom_menu_item_title'] ); ?>
                                </span>
                            </a>
                        <?php else: ?>
                            <div class="mdp-floater-elementor-custom-navigation-text-wrapper">
                                <div class="mdp-floater-elementor-custom-navigation-icon">
                                    <?php Icons_Manager::render_icon( $item['custom_menu_item_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                                </div>
                                <span class="mdp-floater-elementor-custom-navigation-title
                                <?php if ( $settings['display_menu_title'] === 'in-another-block' ): ?> mdp-floater-elementor-custom-navigation-title-absolute <?php endif; ?>">
                                    <?php esc_html_e( $item['custom_menu_item_title'] ); ?>
                                </span>
                            </div>
                        <?php endif; ?>
                    </div>
                <?php endforeach; ?>
            </nav>
        <?php
    }

    /**
     * Set menu arguments.
     *
     * @access private
     *
     * @param $settings
     * @return array
     */

    private function set_menu_arguments( $settings ) {
        $args = array(
            'menu' => $settings['select_menu'],
            'menu_class' => 'mdp-floater-elementor-wp-menu',
            'echo' => false,
            'container' => '',
        );

        //setting class for pointer
        if( $settings['wp_menu_pointer'] !== 'none' ) {
            $args['menu_class'] .= ' '.$settings['wp_menu_pointer'];
        }

        //setting classes for menu position and expand submenu position
        $args['menu_class'] .= ' mdp-floater-wp-menu-expand-position-'.$settings['expand_wp_menu_position_horizontal'];

        if ( $settings['menu_layout'] === 'vertical' ) {
            $args['menu_class'] .= ' mdp-floater-elementor-wp-menu-vertical mdp-floater-wp-menu-expand-position-'.$settings['expand_wp_menu_position_vertical'];
        }

        return $args;
    }

    /**
     * Adding custom filters.
     *
     * @access private
     *
     * @return void
     */
    private function add_filter_floater() {

        //adding custom filter classes on submenu
        add_filter( 'nav_menu_submenu_css_class', [ $this, 'add_sub_menu_classes' ] );

        //adding custom filter classes on links that contains dropdown
        add_filter( 'nav_menu_link_attributes', [ $this, 'add_class_to_items_link' ], 10, 3 );

        //adding custom filter classes on nav menu items
        add_filter( 'nav_menu_css_class', [ $this ,'custom_nav_item_classes' ], 10, 2 );

        //adding custom filter adding menu indicator
        add_filter( 'walker_nav_menu_start_el', [$this, 'adding_menu_indicator'], 10, 4 );

    }

    /**
     * Removing custom filters.
     *
     * @access private
     *
     * @return void
     */
    private function remove_filter_floater() {

        //delete custom filters
        remove_filter( 'nav_menu_link_attributes', [ $this, 'add_class_to_items_link' ] );
        remove_filter( 'nav_menu_submenu_css_class', [ $this, 'add_sub_menu_classes' ] );
        remove_filter( 'nav_menu_css_class', [$this, 'custom_nav_item_classes'] );
        remove_filter( 'walker_nav_menu_start_el', [$this, 'menu_category_item_post_count'] );
        remove_filter( 'walker_nav_menu_start_el', [$this, 'adding_menu_indicator'] );

    }


    /**
     * Adding classes to menu items links
     *
     * @access public
     *
     * @param $atts
     * @param $item
     * @return mixed
     */
    public function add_class_to_items_link( $atts, $item ) {

        // check if the item has children
        $hasChildren = ( in_array( 'menu-item-has-children', $item->classes ) );
        if ( $hasChildren ) {
            // add custom attributes:
            $atts['class'] = 'mdp-floater-elementor-wp-menu-dropdown-link';
            $atts['data-toggle'] = 'dropdown';
            $atts['data-target'] = '#';
        } else {
            $atts['class'] = 'mdp-floater-elementor-wp-menu-link';
        }
        return $atts;

    }

    /**
     * Adding classes to navigation menu items
     *
     * @access public
     *
     * @param $classes
     * @param $item
     * @return mixed
     */
    public function custom_nav_item_classes( $classes, $item ) {

        if ( in_array( 'current-menu-item', $item->classes ) ) {
            $classes[] .= ' mdp-floater-elementor-wp-menu-item-active';
        }

        $classes[] = 'mdp-floater-elementor-wp-menu-item';

        return $classes;
    }

    /**
     * Adding submenu classes method
     *
     * @access public
     *
     * @param $classes
     * @return mixed
     */
    public function add_sub_menu_classes( $classes ) {

        $settings = $this->get_settings_for_display();

        $classes[] = 'mdp-floater-elementor-wp-menu-dropdown';

        if( $settings['wp_submenu_pointer'] != 'none' ) {
            $classes[] .= $settings['wp_submenu_pointer'];
        }

        return $classes;
    }

    /**
     * Adding submenu menu indicator method
     *
     * @access public
     *
     * @param $item_output
     * @param $item
     * @return mixed|string|string[]
     */
    public function adding_menu_indicator( $item_output, $item ) {

        $settings = $this->get_settings_for_display();

        if ( in_array( 'menu-item-has-children', $item->classes ) ) {
            if ( $settings['wp_menu_submenu_indicator'] === 'custom' ) {
                $arrow = '<i class="mdp-floater-wp-menu-submenu-indicator '.esc_attr__( $settings['wp_menu_custom_submenu_indicator']['value'] ).'"></i>';
            } else {
                $arrow = '<i class="mdp-floater-wp-menu-submenu-indicator '.esc_attr__( $settings['wp_menu_submenu_indicator'] ).'"></i>';
            }
            $item_output = str_replace('</a>', '</a>'. $arrow .'', $item_output);
        }
        return $item_output;
    }

    /**
     * WP menu.
     *
     * @param $settings
     *
     * @return void
     */
    private function render_wp_menu( $settings ) {
        if ( !$this->get_menus() ) {
            return;
        }

        // adding filters
        $this->add_filter_floater();

        //setting nav menu
        $menu = wp_nav_menu( $this->set_menu_arguments( $settings ) );

        //removing filters
        $this->remove_filter_floater();

    ?>
        <nav class="mdp-floater-elementor-wp-menu-nav">
            <div class="mdp-floater-elementor-wp-menu-wrapper <?php  esc_html_e( $settings['expand_wp_menu_position_vertical'] )  ?> ">
                <?php echo wp_kses_post( $menu ); ?>
            </div>
        </nav>

    <?php
    }


    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        $settings = $this->get_settings_for_display();
        ?>
        <!-- Start Floater for Elementor WordPress Plugin -->
        <div class="mdp-floater-elementor-box"
        data-menu-displaying="<?php esc_attr_e( $settings['menu_displaying'] ); ?>"
        <?php if ( array_key_exists( 'widescreen', Plugin::$instance->breakpoints->get_active_breakpoints() ) && isset( $settings['menu_displaying_widescreen'] ) ): ?> data-menu-displaying-widescreen="<?php esc_attr_e( $settings['menu_displaying_widescreen'] ); ?>" <?php endif; ?>
        <?php if ( array_key_exists( 'laptop', Plugin::$instance->breakpoints->get_active_breakpoints() ) && isset( $settings['menu_displaying_laptop'] ) ): ?> data-menu-displaying-laptop="<?php esc_attr_e( $settings['menu_displaying_laptop'] ); ?>" <?php endif; ?>
        <?php if ( array_key_exists( 'tablet_extra', Plugin::$instance->breakpoints->get_active_breakpoints() ) && isset( $settings['menu_displaying_tablet_extra'] ) ): ?> data-menu-displaying-tablet-extra="<?php esc_attr_e( $settings['menu_displaying_tablet_extra'] ); ?>" <?php endif; ?>
        <?php if ( array_key_exists( 'mobile_extra', Plugin::$instance->breakpoints->get_active_breakpoints() ) && isset( $settings['menu_displaying_mobile_extra'] ) ): ?> data-menu-displaying-mobile-extra="<?php esc_attr_e( $settings['menu_displaying_mobile_extra'] ); ?>" <?php endif; ?>
        <?php if ( isset( $settings['menu_displaying_mobile'] ) ): ?> data-menu-displaying-mobile="<?php esc_attr_e( $settings['menu_displaying_mobile'] ); ?>" <?php endif; ?>
        <?php if ( isset( $settings['menu_displaying_tablet'] ) ): ?> data-menu-displaying-tablet="<?php esc_attr_e( $settings['menu_displaying_tablet'] ); ?>" <?php endif; ?>
        data-menu-title-hover = "<?php esc_attr_e( $settings['display_menu_title_hover'] ); ?>"
        data-toggle-menu-animation="<?php echo !empty( $settings['toggle_menu_animation'] ) ? esc_attr( $settings['toggle_menu_animation'] ) : ''; ?>"
        data-toggle-menu-animation-duration="<?php echo !empty( $settings['toggle_menu_animation_duration'] ) ? esc_attr( $settings['toggle_menu_animation_duration']['size'] ) : ''; ?>"
        data-toggle-menu-animation-easing="<?php echo !empty( $settings['toggle_menu_animation_easing'] ) ? esc_attr( $settings['toggle_menu_animation_easing'] ) : ''; ?>"
        data-toggle-menu-animation-delay="<?php echo !empty( $settings['toggle_menu_animation_delay'] ) ? esc_attr( $settings['toggle_menu_animation_delay']['size'] ) : ''; ?>"
        data-menu-type="<?php esc_attr_e( $settings['menu_type'] ); ?>"
        data-toggle-button-class="<?php esc_attr_e( $settings['menu_toggle']['value'] ); ?>"
        data-toggle-close-button-class="<?php esc_attr_e( $settings['menu_close_toggle']['value'] ); ?>"
        data-menu-position = "<?php esc_attr_e( $settings['menu_position'] ); ?>"
        >
            <div class="mdp-floater-elementor-navigation-wrapper mdp-floater-elementor-navigation-layout-<?php esc_attr_e( $settings['menu_layout'] ); ?>
                mdp-floater-elementor-navigation-position-<?php esc_attr_e( $settings['menu_position'] ); ?>">
                <?php
                    if ( $settings['menu_type'] === 'custom' ) {
                        $this->render_custom_menu( $settings );
                    } else {
                        $this->render_wp_menu( $settings );
                    }
                ?>
            </div>
        </div>
        <style>
            .elementor-element-<?php esc_html_e( $this->get_id() ); ?> .elementor-widget-empty-icon {
                padding: 0 !important;
            }
        </style>
        <!-- End Floater for Elementor WordPress Plugin -->
	    <?php
        if ( is_admin() ) {
            $widget_hash = substr( hash( 'ripemd160', date('l jS \of F Y h:i:s A') ), rand( 0 , 20 ), 3 ) . rand( 11 , 99 );
            ?>
            <!--suppress JSUnresolvedFunction -->
            <script>
                try {
                    floaterReady<?php esc_attr_e( $widget_hash ); ?>( mdpFloater.addNavigation.bind( mdpFloater ) );
                } catch ( msg ) {
                    const floaterReady<?php esc_attr_e( $widget_hash ); ?> = ( callback ) => {
                        'loading' !== document.readyState ?
                            callback() :
                            document.addEventListener( 'DOMContentLoaded', callback );
                    };
                    floaterReady<?php esc_attr_e( $widget_hash ); ?>( mdpFloater.addNavigation.bind( mdpFloater ) );
                }
            </script>
            <?php

        }
    }
    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/floater';

    }

}
